//Published by Ryan Morrison 7-16-17
//This sketch controls a 7" LCD display to output useful car statistics
//as well as backup sensor information. It takes input from an OBD I2C reader
//and a backup sensor array transceiver to get the data it displays. 

//The Adafruit_RA8875 and Adafruit_GFX libraries used in this sketch were 
//slightly edited so please make sure to download a copy of the edited
//libraries on the instructables page.  

//This code includes a different temperature sensor function. Please make sure
//the temperature sensor test works before running this program. If you are
//using a different sensor than the one that I linked on the instructables (DS18B20), edit
//the read_temperature() function.  


#include <Arduino.h>
#include <Wire.h>
#include <SPI.h>
#include <OBD.h>
#include <Adafruit_GFX.h>       
#include <SD.h>
#include "Adafruit_RA8875.h"
#include <Adafruit_STMPE610.h>
#include <OneWire.h>

//regular fonts
#include <Fonts/FreeSevenSegNumFont.h>
#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeSans18pt7b.h>     
#include <Fonts/FreeSans24pt7b.h>
//bold fonts
#include <Fonts/FreeSansBold12pt7b.h>
#include <Fonts/FreeSansBold18pt7b.h>                          

// Library only supports hardware SPI at this time
// Connect SCLK to UNO Digital #13 (Hardware SPI clock)
// Connect MISO to UNO Digital #12 (Hardware SPI MISO)
// Connect MOSI to UNO Digital #11 (Hardware SPI MOSI)
#define RA8875_INT 8
#define RA8875_CS 10
#define RA8875_RESET 9

//This section controls the colors! If you would like to change the look
//of the display, feel free to edit this. You may put in any RGB565 color.
#define top_background  RA8875_BLACK
#define inner_circles RA8875_BLACK
#define circle_linings RA8875_DGRAY
#define circles RA8875_BLACK
#define middle_background RA8875_BLACK
#define bottom_background RA8875_BLACK
#define bottom_middle RA8875_BLACK
#define top_lining RA8875_BLACK
#define bottom_lining RA8875_DGRAY 
#define bottom_side_lining 0x18E3 //0x18E3
#define linings 0x0969
#define backup_area 0x18C3
#define bright_red RA8875_RED
#define bright_yellow RA8875_YELLOW
#define bright_green RA8875_GREEN
#define dim_yellow 0xA580
#define dim_green 0x1404


//Teensy 3.6 has a built in SD card
const int chipSelect = BUILTIN_SDCARD;

//Variables used for timing purposes so there were no delay functions used
//check out https://learn.adafruit.com/multi-tasking-the-arduino-part-1/using-millis-for-timing for more information
unsigned long timer0 = 0, timer1 = 0, timer2 = 0, timer3 = 0;
unsigned long graytimer0 = 0, graytimer1 = 0, graytimer2 = 0, graytimer3 = 0;

//variables that store the current OBD PID values and sensor values
int ispeed_mph = 0, irpm = 0, iengine_load = 0, icabin_temperature = 0, icoolant_temp = 0;
float fvoltage = 0;

//Variables used to store the previous OBD PID values so that the display only has to update the values that have changed since the
//last iteration of the main loop. This reduces the flickering effect that would occur if the values were changed every time. 
//the previous cases are variables that will be explained in the functions below. 
int iprevious_rpm = 100, ipreviouscase1 = 0, iprevious_mph = 0, ipreviouscase2 = 0, iprevious_eload = 0, ipreviouscase3 = 0;

Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);

//define commonly used x and y values on the display
int horizontal_middle = 480/2;
int vertical_middle = 800/2;
int car_height = 250;
int car_end = 250+5;
int car_width = 110;

//PIN that reads the temperature information 
int temperaturePin=16;
//PIN that reads the transceiver information 
int pin = 14;
//PIN for the LED on the Teensy
int ledPin = 13;
//PIN for dimming purposes (I used analogRead because of the variences in car battery voltage)
int nightPin = A20; //pin 39 or A20
//PIN for engaging backup page (I used analogRead because of the variences in car battery voltage)
int backupPin = A19; //pin 38 or A19

//Varaibles for the current value and previous value of the night and backup pin
//These are used to only change dimming mode or backup mode once
int currentnightPinVal = 0, prevnightPinVal = 2;
int currentbackupPinVal = 0,prevbackupPinVal = 2;

//Variable to store the backlight level
int currentBacklightlevel = 255;

//Variables for the backup transceiver
boolean high_detected = false;
boolean e_activated = false;

//OBD variables
COBDI2C obd;
bool hasMEMS;

//temperature sensor variables
OneWire ds(temperaturePin); 

//2x41 2D array that stores the x (1st row) and y (2nd row) values of the shapes used on the backup page 
//please see the graphic with point numbers on the picture to know which point corresponds to which shape
//[rows][columns]
int pointMatrix[2][41] = {                                                                                                                                                             
    //1   2   3   4     5      6      7      8      9      10    11     12     13     14     15     16     17     18     19     20     21     22     23     24     25     26     27     28     29     30     31     32     33     34     35     36     37     38     39     40 
  {0,368,388,412,432,314+35,357+15,361+15,398   ,402   ,439-15,443-15,486-35,301+35,351+16,355+16,398   ,402   ,445-15,449-15,499-35,281+33,343+12,347+12,398   ,402   ,453-12,457-12,519-33,265+28,292+25,279+30,318+20,322+20,398   ,402   ,478-20,482-20,521-30,508-25,535-28}, //x
  {0,247,253,253,247,225+25,259+05,259+06,265+05,265+05,259+06,259+05,225+25,256+25,285+07,285+ 8,292+05,292+05,285+ 8,285+07,256+25,336-02,364-15,364-14,372-17,372-17,364-14,364-15,336-02,353+32,377+24,409+20,430+02,430+03,440   ,440   ,430+03,430+02,409+20,377+24,353+32}  //y
};





void setup () {

  pinMode(pin, INPUT);
  pinMode(ledPin, OUTPUT);
  
  Serial.begin(9600);
  
  Serial.print("Initializing SD card...");
  if (!SD.begin(chipSelect)) {
    Serial.println("initialization failed!");
    return;
  }
  Serial.println("initialization done.");
  Serial.println("RA8875 start");
  /* Initialise the display using 'RA8875_480x272' or 'RA8875_800x480' */
  if (!tft.begin(RA8875_800x480)) {
    Serial.println("RA8875 Not Found!");
    while (1);
  }

  Serial.println("Found RA8875");
  
  main_page_setup();
  
  read_temperature();
  Serial.println(icabin_temperature);

  obd.begin();
  //hasMEMS = obd.memsInit();

  // initialize OBD-II adapter
  do {
    Serial.println("Init OBD...");
  } while (!obd.init());

}




void loop(){
  
  if(analogRead(nightPin) > 205) currentnightPinVal = HIGH;  //greater than 1V with resistor dividor
  else currentnightPinVal = LOW;
  
  if(analogRead(backupPin) > 205) currentbackupPinVal = HIGH; //greater than 1V with resistor dividor
  else currentbackupPinVal = LOW;


  
  if(currentbackupPinVal == LOW){
    if(prevbackupPinVal != currentbackupPinVal){ //only run setup once
      main_page_setup(); 
      prevbackupPinVal = currentbackupPinVal;
      Serial.println("First page done setting up");
      delay(200);
    }
    main_page_loop();
  }
  else{ //backup PIN is HIGH
    if(prevbackupPinVal != currentbackupPinVal){ //only run setup once
      second_page_setup();
      prevbackupPinVal = currentbackupPinVal;
      Serial.println("Second page done setting up");
      iprevious_mph = 99; //reset the previous stored value so that next time through main loop, speed,rpm,and eload will print
      iprevious_rpm = 99;
      iprevious_eload = 99;
      ipreviouscase1 = 9;
      ipreviouscase2 = 9;
      ipreviouscase3 = 9;
      
    }
    second_page_loop(1); //pass in 1 for feet, 0 for meters
  }

  //Full brightness 
  if((currentnightPinVal == LOW) && prevnightPinVal != currentnightPinVal){
    delay(300);
    currentBacklightlevel = 255;
    tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
    tft.PWM1out(currentBacklightlevel);
    prevnightPinVal = currentnightPinVal;
    Serial.print("Changed the PWM to ");
    Serial.println(currentBacklightlevel);
  }
  //Dimmed mode
  else if( prevnightPinVal != currentnightPinVal){
    delay(300);
    currentBacklightlevel= 50;
    tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
    tft.PWM1out(currentBacklightlevel);
    prevnightPinVal = currentnightPinVal;
    Serial.print("Changed the PWM to ");
    Serial.println(currentBacklightlevel);
  }
  
}    






////// Main Page //////
void main_page_setup(){
  turn_on();
  tft.graphicsMode();
  

  //Draw all of the graphics on the main screen
  tft.fillScreen(middle_background);
  tft.fillEllipse(vertical_middle, 0, 452, 52, top_lining);
  tft.fillEllipse(vertical_middle, 0, 450, 50, top_background);

  tft.fillRect(0, 480-87, 800, 85, bottom_side_lining);
  tft.fillRect(0, 480-85, 800, 85, bottom_background);
  
  tft.fillCircle(vertical_middle, horizontal_middle, 175+4, circle_linings);
  tft.fillCircle(vertical_middle-285, horizontal_middle, 100+4, circle_linings);
  tft.fillCircle(vertical_middle+285, horizontal_middle, 100+4, circle_linings);
  tft.fillCircle(vertical_middle, horizontal_middle, 175, circles);
  tft.fillCircle(vertical_middle-285, horizontal_middle, 100, circles);
  tft.fillCircle(vertical_middle+285, horizontal_middle, 100, circles);
  tft.fillCircle(vertical_middle, horizontal_middle, 175-10, inner_circles);
  tft.fillCircle(vertical_middle-285, horizontal_middle, 100-10, inner_circles);
  tft.fillCircle(vertical_middle+285, horizontal_middle, 100-10, inner_circles);
  
  tft.fillRoundRect(vertical_middle-100-2, 400-2, 200+4, 100, 15, bottom_lining);
  tft.fillRoundRect(vertical_middle-100, 400, 200, 100, 15, bottom_middle);


  //Print the text that doesn't change on the main page
  tft.graphicsMode();
  tft.setTextSize(1);
  tft.setFont(&FreeSans12pt7b);
  tft.setCursor(115-18,horizontal_middle+60);
  tft.print("rpm"); //prints from bottom left
  tft.setCursor(800-115-35,horizontal_middle+60);
  tft.print("% load"); //prints from bottom left
  
  tft.setFont(&FreeSans24pt7b);
  tft.setCursor(vertical_middle-45,horizontal_middle+100);
  tft.print("mph"); //prints from bottom left

  tft.setFont(&FreeSans12pt7b);
  tft.setCursor(10,450);
  tft.print("Voltage:"); //prints from bottom left
  tft.setCursor(800-115 - 55,450);
  tft.print("Coolant:"); //prints from bottom left
  tft.setCursor(vertical_middle-70,450-15);
  tft.print("Temperature"); //prints from bottom left
}

////// Main Page //////
void main_page_loop(){
  //every 100 milliseconds
  if(millis() - timer0 > 100UL){

    readPIDs(); //read all car variables from OBD port

    //print speed
    print_int(ispeed_mph,iprevious_mph,ipreviouscase2, vertical_middle,horizontal_middle+15,60,100,2); 

    //round rpm number to the nearest 100
    if( abs(irpm - iprevious_rpm) > 49 || irpm == 0){ 
      int round_factor = 100;
      int rem = irpm%round_factor;
      int print_rpm;
      if(rem < (round_factor/2)) print_rpm = irpm - rem;
      else if(irpm + (round_factor - rem) > 0){
        print_rpm = irpm + (round_factor - rem);
      }
      else print_rpm = 0;
      //print out the rpm
      print_rpm_int(print_rpm,iprevious_rpm,ipreviouscase1,115,horizontal_middle+15,30,50,1);
    }

    //print engine load
    print_int(iengine_load,iprevious_eload,ipreviouscase3,800-115,horizontal_middle+15,30,50,1); 
      
    timer0 = millis();
  }

  //Every 10 seconds
  if(millis() - timer1 > 10000UL){
    fvoltage = obd.getVoltage();
    read_temperature();
    print_float_voltage(fvoltage, 100, 450);
    print_int_coolant_temp(icoolant_temp, 720, 450);
    print_int_cabin_temperature(icabin_temperature,vertical_middle-35,450+20);
    timer1 = millis();
  }
}


////// Second Page //////
void second_page_setup(){
  tft.displayOn(false);
  tft.GPIOX(true);      // Enable TFT - display enable tied to GPIOX
  tft.graphicsMode();                 // go back to graphics mode
  tft.fillScreen(RA8875_BLACK);
  tft.graphicsMode(); 
  bmpDraw("car.bmp", vertical_middle - car_width/2, 5);
  turn_on();
  
  initialize_backup_area();
}

////// Second Page //////
void second_page_loop(int meter0_or_feet_1){
  byte address;
  byte value;
  byte pulse_value;
  float current_value_for_unit_selected;
  float meter_value;
  float feet_value;
  unsigned long pulse_length;
  byte sensorValue[24];
  float Avalue,Bvalue,Cvalue,Dvalue,minimum_value;

  //look for starter pulse
  pulse_length = pulseIn(pin, HIGH);
  if(pulse_length > 500){
    int i;
    //read in the next 17 values
    for (i = 0; i < 17; i = i + 1) {
      pulse_length = pulseIn(pin, HIGH);
      if (pulse_length > 20 && pulse_length < 120) {
        pulse_value = B0;
        digitalWrite(ledPin, LOW);
      }
    else{
      pulse_value = B1;
      digitalWrite(ledPin, HIGH);
    }
    sensorValue[i] = pulse_value;
    }
  
    //determine if A,B,C, or D sensor and calculate value
    address = (sensorValue[6] << 2) + (sensorValue[7] << 1) + sensorValue[8];
    value = (sensorValue[9] << 7) + (sensorValue[10] << 6) + (sensorValue[11] << 5) + (sensorValue[12] << 4) + (sensorValue[13] << 3) + (sensorValue[14] << 2) + (sensorValue[15] << 1) + sensorValue[16];
    
    //convert to meters
    if(address == 0 || address == 2 || address == 4 || address == 6){
      meter_value = float(value)*0.005;
    }
    else if(address == 1 || address == 3 || address == 5 || address == 7){
      meter_value = (float(value)+256)*0.005;
    }

    //checks if user wants feet or meters
    if(meter0_or_feet_1 == 0){
      current_value_for_unit_selected = meter_value;
    }
    else{
      feet_value = meter_value*3.28084;
      current_value_for_unit_selected = feet_value;
    }

    if(value == 254){
      update_out_of_range(address);
    }
    //Assigns calculated value to certain sensor based off of address
    else if(address == 0 || address == 1){
      Avalue = current_value_for_unit_selected;
      update_A(Avalue,meter0_or_feet_1);
    }
    else if(address == 2 || address == 3){
      Bvalue = current_value_for_unit_selected;
      update_B(Bvalue,meter0_or_feet_1);
    }
    else if(address == 4 || address == 5){
      Cvalue = current_value_for_unit_selected;
      update_C(Cvalue,meter0_or_feet_1);
    }
    else if (address == 6 || address == 7){
      //Serial.print("D: ");
      Dvalue = current_value_for_unit_selected;
      update_D(Dvalue,meter0_or_feet_1);
    }
    
    minimum_value = min(Avalue,min(Bvalue,min(Cvalue,Dvalue)));
    if((minimum_value < 0.34 && meter0_or_feet_1 == 0) || (minimum_value < (0.34*3.28084) && meter0_or_feet_1 == 1)){
      e_activated = true;
    }
    if(e_activated){
      //Serial.print(minimum_value);
      //Serial.println(" E");
      e_activated = false;
    }
//    else{
//      Serial.println(minimum_value);
//    }
  }
}






//Supporting Functions
void turn_on(){
  tft.displayOn(true);
  tft.GPIOX(true);      // Enable TFT - display enable tied to GPIOX
  tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
  tft.PWM1out(currentBacklightlevel);
}

// This function opens a Windows Bitmap (BMP) file and
// displays it at the given coordinates.  It's sped up
// by reading many pixels worth of data at a time
// (rather than pixel by pixel).  Increasing the buffer
// size takes more of the Arduino's precious RAM but
// makes loading a little faster.  20 pixels seems a
// good balance.

#define BUFFPIXEL 85


void bmpDraw(char *filename, int x, int y) {
  File     bmpFile;
  int      bmpWidth, bmpHeight;   // W+H in pixels
  uint8_t  bmpDepth;              // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;        // Start of image data in file
  uint32_t rowSize;               // Not always = bmpWidth; may have padding
  uint8_t  sdbuffer[3*BUFFPIXEL]; // pixel in buffer (R+G+B per pixel)
  uint16_t lcdbuffer[BUFFPIXEL];  // pixel out buffer (16-bit per pixel)
  uint8_t  buffidx = sizeof(sdbuffer); // Current position in sdbuffer
  boolean  goodBmp = false;       // Set to true on valid header parse
  boolean  flip    = true;        // BMP is stored bottom-to-top
  int      w, h, row, col;
  uint8_t  r, g, b;
  uint32_t pos = 0, startTime = millis();
  uint8_t  lcdidx = 0;
  boolean  first = true;

  if((x >= tft.width()) || (y >= tft.height())) return;

  Serial.println();
  Serial.print(F("Loading image '"));
  Serial.print(filename);
  Serial.println('\'');

  // Open requested file on SD card
  if ((bmpFile = SD.open(filename)) == NULL) {
    Serial.println(F("File not found"));
    return;
  }

  // Parse BMP header
  if(read16(bmpFile) == 0x4D42) { // BMP signature
    Serial.println(F("File size: ")); 
    Serial.println(read32(bmpFile));
    (void)read32(bmpFile); // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile); // Start of image data
    Serial.print(F("Image Offset: ")); 
    Serial.println(bmpImageoffset, DEC);

    // Read DIB header
    Serial.print(F("Header size: ")); 
    Serial.println(read32(bmpFile));
    bmpWidth  = read32(bmpFile);
    bmpHeight = read32(bmpFile);

    if(read16(bmpFile) == 1) { // # planes -- must be '1'
      bmpDepth = read16(bmpFile); // bits per pixel
      Serial.print(F("Bit Depth: ")); 
      Serial.println(bmpDepth);
      if((bmpDepth == 24) && (read32(bmpFile) == 0)) { // 0 = uncompressed
        goodBmp = true; // Supported BMP format -- proceed!
        Serial.print(F("Image size: "));
        Serial.print(bmpWidth);
        Serial.print('x');
        Serial.println(bmpHeight);

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        // If bmpHeight is negative, image is in top-down order.
        // This is not canon but has been observed in the wild.
        if(bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip      = false;
        }

        // Crop area to be loaded
        w = bmpWidth;
        h = bmpHeight;
        if((x+w-1) >= tft.width())  w = tft.width()  - x;
        if((y+h-1) >= tft.height()) h = tft.height() - y;

        // Set TFT address window to clipped image bounds

        for (row=0; row<h; row++) { // For each scanline...
          // Seek to start of scan line.  It might seem labor-
          // intensive to be doing this on every line, but this
          // method covers a lot of gritty details like cropping
          // and scanline padding.  Also, the seek only takes
          // place if the file position actually needs to change
          // (avoids a lot of cluster math in SD library).
          if(flip) // Bitmap is stored bottom-to-top order (normal BMP)
            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
          else     // Bitmap is stored top-to-bottom
          pos = bmpImageoffset + row * rowSize;
          if(bmpFile.position() != pos) { // Need seek?
            bmpFile.seek(pos);
            buffidx = sizeof(sdbuffer); // Force buffer reload
          }

          for (col=0; col<w; col++) { // For each column...
            // Time to read more pixel data?
            if (buffidx >= sizeof(sdbuffer)) { // Indeed
              // Push LCD buffer to the display first
              if(lcdidx > 0) {
                tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
                lcdidx = 0;
                first  = false;
              }

              bmpFile.read(sdbuffer, sizeof(sdbuffer));
              buffidx = 0; // Set index to beginning
            }

            // Convert pixel from BMP to TFT format
            b = sdbuffer[buffidx++];
            g = sdbuffer[buffidx++];
            r = sdbuffer[buffidx++];
            lcdbuffer[lcdidx] = color565(r,g,b);
            tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
          } // end pixel

        } // end scanline

        // Write any remaining data to LCD
        if(lcdidx > 0) {
          tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
        } 

        Serial.print(F("Loaded in "));
        Serial.print(millis() - startTime);
        Serial.println(" ms");

      } // end goodBmp
    }
  }

  bmpFile.close();
  if(!goodBmp) Serial.println(F("BMP format not recognized."));

}

// These read 16- and 32-bit types from the SD card file.
// BMP data is stored little-endian, Arduino is little-endian too.
// May need to reverse subscript order if porting elsewhere.

uint16_t read16(File f) {
  uint16_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read(); // MSB
  return result;
}

uint32_t read32(File f) {
  uint32_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read();
  ((uint8_t *)&result)[2] = f.read();
  ((uint8_t *)&result)[3] = f.read(); // MSB
  return result;
}

uint16_t color565(uint8_t r, uint8_t g, uint8_t b) {
  return ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | (b >> 3);
}

byte decToBcd(byte val){
  // Convert normal decimal numbers to binary coded decimal
  return ( (val/10*16) + (val%10) );
}


//prints light gray area to show where backup sensors can detect
//depending on which sensor is currently out of range
void update_out_of_range(byte address){
  //determine if out of range
  if((address == 0 || address == 1)&& (millis() - graytimer0 > 500UL)){  //A
    tft.graphicsMode();
    tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],backup_area);
    tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],backup_area);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],backup_area);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, backup_area);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, backup_area); 
    graytimer0 = millis();
  }
  else if((address == 2 || address == 3)&& (millis() - graytimer1 > 500UL)){ //B  
    tft.graphicsMode();
    tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
    tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
    graytimer1 = millis();
  }
  else if((address == 4 || address == 5)&& (millis() - graytimer2 > 500UL)){  //C
    tft.graphicsMode();
    tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);
    tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);
    graytimer2 = millis();
  }
  
  else if((address == 6 || address == 7)&& (millis() - graytimer3 > 500UL)){  //D
    tft.graphicsMode();
    tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],backup_area);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, backup_area);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, backup_area);
    graytimer3 = millis();
  }
}


//Prints out either green, yellow, or red depending on how close an object is to the sensor
//if the object is in the green or yellow range, print the green and yellow in a dimmer color. 
void update_A(float Avalue, int meter0_or_feet_1){
  //Serial.println(Avalue);
  if((Avalue < 0.34 && meter0_or_feet_1 == 0) || (Avalue < (0.34*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-125,480-30);
    tft.textTransparent(RA8875_RED);
    if(meter0_or_feet_1 == 0) tft.textWrite("<.3");
    else tft.textWrite("<1");
    tft.graphicsMode();
    //draw all 3 regions bright
    tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],bright_red);
    tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],bright_red);
    tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],bright_yellow);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],bright_yellow);
    tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],bright_green);
    tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],bright_green);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],bright_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, bright_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, bright_green);
  }
  else if((Avalue < 0.8 && meter0_or_feet_1 == 0) || (Avalue < (0.8*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-125,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Avalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 2 regions dim 
    tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],dim_yellow);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],dim_yellow);
    tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],dim_green);
    tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],dim_green);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],dim_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, dim_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, dim_green);
  }
  else if((Avalue < 1.52 && meter0_or_feet_1 == 0) || (Avalue < (1.52*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-125,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Avalue, 3, 1, floatstring));
    tft.graphicsMode();
      //draw all 1 region dim 
    tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],dim_green);
    tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],dim_green);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],dim_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, dim_green);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, dim_green); 
  }
  else{
    tft.graphicsMode();
    tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
    tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
    tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],backup_area);
    tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],backup_area);
    tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],backup_area);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, backup_area);
    tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, backup_area);
  }
  graytimer0 = millis();
}



//Prints out either green, yellow, or red depending on how close an object is to the sensor
//if the object is in the green or yellow range, print the green and yellow in a dimmer color.
void update_B(float Bvalue, int meter0_or_feet_1){
  //Serial.println(Bvalue);
    
  if((Bvalue < 0.34 && meter0_or_feet_1 == 0) || (Bvalue < (0.34*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-58,480-30);
    tft.textTransparent(RA8875_RED);
    if(meter0_or_feet_1 == 0) tft.textWrite("<.3");
    else tft.textWrite("<1");
    tft.graphicsMode();
    //draw all 3 regions bright
    tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],bright_red);
    tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],bright_red);
    tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],bright_yellow);
    tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],bright_yellow);
    tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],bright_green);
    tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],bright_green);
  }
  else if((Bvalue < 0.8 && meter0_or_feet_1 == 0) || (Bvalue < (0.8*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-58,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Bvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 2 regions dim
    tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],dim_yellow);
    tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],dim_yellow);
    tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],dim_green);
    tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],dim_green); 
  }
  else if((Bvalue < 1.52 && meter0_or_feet_1 == 0) || (Bvalue < (1.52*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle-58,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Bvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 1 region dim
    tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],dim_green);
    tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],dim_green); 
  }
  else{
    tft.graphicsMode();
    tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
    tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
    tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
    tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
  }
  graytimer1 = millis();
}



//Prints out either green, yellow, or red depending on how close an object is to the sensor
//if the object is in the green or yellow range, print the green and yellow in a dimmer color.
void update_C(float Cvalue, int meter0_or_feet_1){
  //Serial.println(Cvalue);
    
  if((Cvalue < 0.34 && meter0_or_feet_1 == 0) || (Cvalue < (0.34*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+8,480-30);
    tft.textTransparent(RA8875_RED);
    if(meter0_or_feet_1 == 0) tft.textWrite("<.3");
    else tft.textWrite("<1");
    tft.graphicsMode();
    //draw all 3 regions bright
    tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],bright_red);
    tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],bright_red);
    tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],bright_yellow);
    tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],bright_yellow);
    tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],bright_green);
    tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],bright_green);
  }
  else if((Cvalue < 0.8 && meter0_or_feet_1 == 0) || (Cvalue < (0.8*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+8,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Cvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 2 regions dim
    tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],dim_yellow);
    tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],dim_yellow);
    tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],dim_green);
    tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],dim_green); 
  }
  else if((Cvalue < 1.52 && meter0_or_feet_1 == 0) || (Cvalue < (1.52*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+8,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Cvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 1 region dim
    tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],dim_green);
    tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],dim_green); 
  }
  else{
    tft.graphicsMode();
    tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
    tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
    tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);
    tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);
  }
  graytimer2 = millis();
}

    

//Prints out either green, yellow, or red depending on how close an object is to the sensor
//if the object is in the green or yellow range, print the green and yellow in a dimmer color.
void update_D(float Dvalue, int meter0_or_feet_1){
  //Serial.println(Dvalue);
      
  if((Dvalue < 0.34 && meter0_or_feet_1 == 0) || (Dvalue < (0.34*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+75,480-30);
    tft.textTransparent(RA8875_RED);
    if(meter0_or_feet_1 == 0) tft.textWrite("<.3");
    else tft.textWrite("<1");
    tft.graphicsMode();
    //draw all 3 regions bright
    tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],bright_red);
    tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],bright_red);
    tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],bright_yellow);
    tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],bright_yellow);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],bright_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],bright_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],bright_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, bright_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, bright_green);
  }
  else if((Dvalue < 0.8 && meter0_or_feet_1 == 0) || (Dvalue < (0.8*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+75,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Dvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 2 regions dim 
    tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],dim_yellow);
    tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],dim_yellow);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],dim_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],dim_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],dim_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, dim_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, dim_green);
  }
  else if((Dvalue < 1.52 && meter0_or_feet_1 == 0) || (Dvalue < (1.52*3.28084) && meter0_or_feet_1 == 1)){
    tft.graphicsMode();
    tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    tft.textMode();
    char floatstring[3];
    tft.textEnlarge(1);
    tft.textSetCursor(vertical_middle+75,480-30);
    tft.textTransparent(RA8875_WHITE);
    tft.textWrite(dtostrf(Dvalue, 3, 1, floatstring));
    tft.graphicsMode();
    //draw all 1 region dim
    tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],dim_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],dim_green);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],dim_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, dim_green);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, dim_green); 
  }
  else{
    tft.graphicsMode();
    tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
    //draw all 3 regions gray
    tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
    tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],backup_area);
    tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],backup_area);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, backup_area);
    tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, backup_area);
    
  }
  graytimer3 = millis();
}

void initialize_backup_area(){
  tft.fillTriangle(pointMatrix[0][5],pointMatrix[1][5],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
  tft.fillTriangle(pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][13],pointMatrix[1][13],backup_area);
  tft.fillTriangle(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
  tft.fillTriangle(pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][15],pointMatrix[1][15],backup_area);
  tft.fillTriangle(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
  tft.fillTriangle(pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][17],pointMatrix[1][17],backup_area);
  tft.fillTriangle(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);
  tft.fillTriangle(pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][12],pointMatrix[1][12],pointMatrix[0][19],pointMatrix[1][19],backup_area);

  tft.fillTriangle(pointMatrix[0][13],pointMatrix[1][13],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
  tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][14],pointMatrix[1][14],pointMatrix[0][21],pointMatrix[1][21],backup_area);
  tft.fillTriangle(pointMatrix[0][15],pointMatrix[1][15],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
  tft.fillTriangle(pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][16],pointMatrix[1][16],pointMatrix[0][23],pointMatrix[1][23],backup_area);
  tft.fillTriangle(pointMatrix[0][17],pointMatrix[1][17],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
  tft.fillTriangle(pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][18],pointMatrix[1][18],pointMatrix[0][25],pointMatrix[1][25],backup_area);
  tft.fillTriangle(pointMatrix[0][19],pointMatrix[1][19],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);
  tft.fillTriangle(pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][20],pointMatrix[1][20],pointMatrix[0][27],pointMatrix[1][27],backup_area);

  
  tft.fillTriangle(pointMatrix[0][21],pointMatrix[1][21],pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][29],pointMatrix[1][29],backup_area);
  tft.fillTriangle(pointMatrix[0][29],pointMatrix[1][29],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][22],pointMatrix[1][22],backup_area);
  tft.fillTriangle(pointMatrix[0][22],pointMatrix[1][22],pointMatrix[0][31],pointMatrix[1][31],pointMatrix[0][32],pointMatrix[1][32],backup_area);
  tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 0, backup_area);
  tft.fillCurve(pointMatrix[0][30], pointMatrix[1][30], 28, 28, 1, backup_area);
  
  tft.fillTriangle(pointMatrix[0][23],pointMatrix[1][23],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
  tft.fillTriangle(pointMatrix[0][34],pointMatrix[1][34],pointMatrix[0][24],pointMatrix[1][24],pointMatrix[0][33],pointMatrix[1][33],backup_area);
  
  tft.fillTriangle(pointMatrix[0][25],pointMatrix[1][25],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);
  tft.fillTriangle(pointMatrix[0][36],pointMatrix[1][36],pointMatrix[0][26],pointMatrix[1][26],pointMatrix[0][35],pointMatrix[1][35],backup_area);

  tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][37],pointMatrix[1][37],pointMatrix[0][38],pointMatrix[1][38],backup_area);
  tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][38],pointMatrix[1][38],pointMatrix[0][40],pointMatrix[1][40],backup_area);
  tft.fillTriangle(pointMatrix[0][27],pointMatrix[1][27],pointMatrix[0][28],pointMatrix[1][28],pointMatrix[0][40],pointMatrix[1][40],backup_area);
  tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 2, backup_area);
  tft.fillCurve(pointMatrix[0][39], pointMatrix[1][39], 28, 28, 3, backup_area);

  tft.drawLine(pointMatrix[0][6],pointMatrix[1][6],pointMatrix[0][32],pointMatrix[1][32],backup_area);
  tft.drawLine(pointMatrix[0][6]+1,pointMatrix[1][6],pointMatrix[0][32]+1,pointMatrix[1][32],backup_area);
  tft.drawLine(pointMatrix[0][6]+2,pointMatrix[1][6],pointMatrix[0][32]+2,pointMatrix[1][32],backup_area);
  tft.drawLine(pointMatrix[0][7]-2,pointMatrix[1][7],pointMatrix[0][33]-2,pointMatrix[1][33],backup_area);
  tft.drawLine(pointMatrix[0][7]-1,pointMatrix[1][7],pointMatrix[0][33]-1,pointMatrix[1][33],backup_area);
  tft.drawLine(pointMatrix[0][7],pointMatrix[1][7],pointMatrix[0][33],pointMatrix[1][33],backup_area);

  tft.drawLine(pointMatrix[0][8],pointMatrix[1][8],pointMatrix[0][34],pointMatrix[1][34],backup_area);
  tft.drawLine(pointMatrix[0][8]+1,pointMatrix[1][8],pointMatrix[0][34]+1,pointMatrix[1][34],backup_area);
  tft.drawLine(pointMatrix[0][8]+2,pointMatrix[1][8],pointMatrix[0][34]+2,pointMatrix[1][34],backup_area);
  tft.drawLine(pointMatrix[0][9]-2,pointMatrix[1][9],pointMatrix[0][35]-2,pointMatrix[1][35],backup_area);
  tft.drawLine(pointMatrix[0][9]-1,pointMatrix[1][9],pointMatrix[0][35]-1,pointMatrix[1][35],backup_area);
  tft.drawLine(pointMatrix[0][9],pointMatrix[1][9],pointMatrix[0][35],pointMatrix[1][35],backup_area);

  tft.drawLine(pointMatrix[0][10],pointMatrix[1][10],pointMatrix[0][36],pointMatrix[1][36],backup_area);
  tft.drawLine(pointMatrix[0][10]+1,pointMatrix[1][10],pointMatrix[0][36]+1,pointMatrix[1][36],backup_area);
  tft.drawLine(pointMatrix[0][10]+2,pointMatrix[1][10],pointMatrix[0][36]+2,pointMatrix[1][36],backup_area);
  tft.drawLine(pointMatrix[0][11]-2,pointMatrix[1][11],pointMatrix[0][37]-2,pointMatrix[1][37],backup_area);
  tft.drawLine(pointMatrix[0][11]-1,pointMatrix[1][11],pointMatrix[0][37]-1,pointMatrix[1][37],backup_area);
  tft.drawLine(pointMatrix[0][11],pointMatrix[1][11],pointMatrix[0][37],pointMatrix[1][37],backup_area);

  //initialize text bottom of display for backup 
  tft.fillRect(vertical_middle-125-3, 480-28, 55, 30, RA8875_WHITE);
  tft.fillRect(vertical_middle-125-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
  tft.fillRect(vertical_middle-58-3, 480-28, 55, 30, RA8875_WHITE);
  tft.fillRect(vertical_middle-58-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
  tft.fillRect(vertical_middle+8-3, 480-28, 55, 30, RA8875_WHITE);
  tft.fillRect(vertical_middle+8-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
  tft.fillRect(vertical_middle+75-3, 480-28, 55, 30, RA8875_WHITE);
  tft.fillRect(vertical_middle+75-3+1, 480-28+1, 55-2, 28-2, RA8875_BLACK);
  tft.textMode();
  char floatstring[3];
  float floatingpt = 3.3;
  tft.textEnlarge(1);
  tft.textSetCursor(vertical_middle-125,480-30);
  tft.textTransparent(RA8875_WHITE);
  tft.textWrite(dtostrf(floatingpt, 3, 1, floatstring));
  tft.textSetCursor(vertical_middle-58,480-30);
  tft.textTransparent(RA8875_WHITE);
  tft.textWrite(dtostrf(floatingpt, 3, 1, floatstring));
  tft.textSetCursor(vertical_middle+8,480-30);
  tft.textTransparent(RA8875_WHITE);
  tft.textWrite(dtostrf(floatingpt, 3, 1, floatstring));
  tft.textSetCursor(vertical_middle+75,480-30);
  tft.textTransparent(RA8875_WHITE);
  tft.textWrite(dtostrf(floatingpt, 3, 1, floatstring));
}



//prints the voltage
void print_float_voltage(float fnumber, int x, int y){
  tft.graphicsMode();
  tft.setTextSize(1);
  tft.setFont(&FreeSansBold12pt7b);
  char floatstring[7] = " ";
  memset(&floatstring[0], 0, sizeof(floatstring));
  dtostrf(fnumber, 7, 1, floatstring);
  if(fnumber < 100.0){
    floatstring[0] = floatstring[3]; 
    floatstring[1] = floatstring[4];
    floatstring[2] = floatstring[5];
    floatstring[3] = floatstring[6];
    floatstring[4] = ' ';
    floatstring[5] = 'V';
    floatstring[6] = ' ';
  }
  else{
    floatstring[0] = floatstring[2]; 
    floatstring[1] = floatstring[3];
    floatstring[2] = floatstring[4];
    floatstring[3] = floatstring[5];
    floatstring[4] = floatstring[6];
    floatstring[5] = ' ';
    floatstring[6] = 'V';
  }
  tft.fillRect(x, y-20, 100, 25, bottom_background); 
  tft.setCursor(x,y);
  tft.print(floatstring);
  
}



//Intelegently prints a font with a fixed width and height up to 9999 to TFT
//center is at bottom of printed text
//intelegent printing means just print necessary digits and not all of them every time the 
//function is called

//There are 7 different "cases" that correspond to different x and y offsets of the integer.
//Basically, a number such as 15 is a different case than 20 because the 1 in 15 is smaller in width than 20
//This allows the centering of the text at the input  vertical_center and horizontal_center.
void print_int(int ispeed,int & iprevious_mph, int & ipreviouscase, int vertical_center,int horizontal_center,int fixed_width, int fixed_height,int textSize){
  int icurcase;
  int num_1_offset = textSize * 10;
  char intstring[5];
  char prevstring[5];
  String tempstr = String(ispeed);
  String prevstr = String(iprevious_mph);
  tempstr.toCharArray(intstring,5);
  prevstr.toCharArray(prevstring,5);

  tft.graphicsMode();
  tft.setTextSize(textSize);
  tft.setFont(&FreeSevenSegNumFont);

  if(ispeed > 1999){ 
    icurcase = 1;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width*2) ,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center-fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width ,horizontal_center);
      tft.print(intstring[1]);
      //tft.fillRect(vertical_center , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center ,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
      //tft.fillRect(vertical_center+fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center+fixed_width,horizontal_center);
      tft.print(intstring[3]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){ //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width*2) ,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width ,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }
      else if(prevstring[1] != intstring[1]) { //print 1 number
        tft.fillRect(vertical_center-fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width ,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }
      else if(prevstring[2] != intstring[2]) { //print 1 number
        tft.fillRect(vertical_center , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      } 
      else if(prevstring[3] != intstring[3]) { //print 1 number
        tft.fillRect(vertical_center+fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }    
    }
    
  }
  else if(ispeed > 999 && ispeed <= 1999){ 
    icurcase = 2;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width*2)-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
      //tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-num_1_offset,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
      //tft.fillRect(vertical_center+fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[3]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){ //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width*2)-num_1_offset,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }
      else if(prevstring[1] != intstring[1]) { //print 1 number
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }
      else if(prevstring[2] != intstring[2]) { //print 1 number
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
        tft.fillRect(vertical_center+fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      } 
      else if(prevstring[3] != intstring[3]) { //print 1 number
        tft.fillRect(vertical_center+fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[3]); //prints from bottom left
      }    
    }
    
  }
  else if(ispeed > 199){ 
    icurcase = 3;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-(fixed_width + fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width + fixed_width/2) ,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center-(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2) ,horizontal_center);
      tft.print(intstring[1]);
      //tft.fillRect(vertical_center+(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center+(fixed_width/2) ,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-(fixed_width + fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width + fixed_width/2) ,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width/2) ,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center+(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+(fixed_width/2) ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
      }
      else if(prevstring[1] != intstring[1]) { //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width/2) ,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center+(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+(fixed_width/2) ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
      }
      else if(prevstring[2] != intstring[2]) { //print 1 number
        tft.fillRect(vertical_center+(fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+(fixed_width/2) ,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
      }   
    }
    
  }

  else if(ispeed > 99 && ispeed <= 199){ 
    icurcase = 4;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-(fixed_width + fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width + fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center-(fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
      //tft.fillRect(vertical_center+(fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center+(fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
    }
    else{
      if(prevstring[1] != intstring[1]){ //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width/2)-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
        tft.fillRect(vertical_center+(fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+(fixed_width/2)-num_1_offset,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
      }
      else if(prevstring[2] != intstring[2]) { //print 1 number
        tft.fillRect(vertical_center+(fixed_width/2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center+(fixed_width/2)-num_1_offset,horizontal_center);
        tft.print(intstring[2]); //prints from bottom left
      }   
    }
    
  }
  else if(ispeed > 9 && ispeed <= 19){ 
    icurcase = 5;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]){
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }   
    }
    
  }
  else if(ispeed > 19){ 
    icurcase = 6;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-fixed_width, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width,horizontal_center);
      tft.print(intstring[0]);
      //tft.fillRect(vertical_center, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center,horizontal_center);
      tft.print(intstring[1]);
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-fixed_width, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]){
        tft.fillRect(vertical_center, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center,horizontal_center);
        tft.print(intstring[1]);
      }   
    }
  }
  else{ 
    icurcase = 7;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      //tft.fillRect(vertical_center-(fixed_width/2), horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2),horizontal_center);
      tft.print(intstring[0]);
    }
    else if(prevstring[0] != intstring[0]){
      tft.fillRect(vertical_center-(fixed_width/2), horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2),horizontal_center);
      tft.print(intstring[0]);
    }
  }


  ipreviouscase = icurcase;
  iprevious_mph = ispeed;
}


//This function is a modified print_int function from above
//It only prints a number that has been rounded to the nearest 100
void print_rpm_int(int ispeed,int & iprevious_mph, int & ipreviouscase, int vertical_center,int horizontal_center,int fixed_width, int fixed_height,int textSize){
  int icurcase;
  int num_1_offset = textSize * 10;
  char intstring[5];
  char prevstring[5];
  String tempstr = String(ispeed);
  String prevstr = String(iprevious_mph);
  tempstr.toCharArray(intstring,5);
  prevstr.toCharArray(prevstring,5);

  tft.graphicsMode();
  tft.setTextSize(textSize);
  tft.setFont(&FreeSevenSegNumFont);

  if(ispeed > 1999){ 
    icurcase = 1;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width*2) ,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center-fixed_width ,horizontal_center);
      tft.print(intstring[1]);
      tft.setCursor(vertical_center ,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
      tft.setCursor(vertical_center+fixed_width,horizontal_center);
      tft.print(intstring[3]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){ //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width*2) ,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width ,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]) { //print 1 number
        tft.fillRect(vertical_center-fixed_width , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width ,horizontal_center);
        tft.print(intstring[1]);
      }   
    }  
  }
  else if(ispeed > 999 && ispeed <= 1999){ 
    icurcase = 2;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width*2)-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
      tft.setCursor(vertical_center-num_1_offset,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
      tft.setCursor(vertical_center+fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[3]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){ //print 2 numbers
        tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width*2)-num_1_offset,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]) { //print 1 number
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }    
    }
    
  }
  else if(ispeed > 199){ 
    icurcase = 3;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2) , horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width + fixed_width/2) ,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center-(fixed_width/2) ,horizontal_center);
      tft.print(intstring[1]);
      tft.setCursor(vertical_center+(fixed_width/2) ,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-(fixed_width + fixed_width/2) , horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-(fixed_width + fixed_width/2) ,horizontal_center);
        tft.print(intstring[0]);

      } 
    }
    
  }

  else if(ispeed > 99 && ispeed <= 199){ 
    icurcase = 4;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width + fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center-(fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
      tft.setCursor(vertical_center+(fixed_width/2)-num_1_offset,horizontal_center);
      tft.print(intstring[2]); //prints from bottom left
    }   
  }
  else if(ispeed > 9 && ispeed <= 19){ 
    icurcase = 5;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center-num_1_offset,horizontal_center);
      tft.print(intstring[1]);
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-fixed_width-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width-num_1_offset,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]){
        tft.fillRect(vertical_center-num_1_offset, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-num_1_offset,horizontal_center);
        tft.print(intstring[1]);
      }   
    }
    
  }
  else if(ispeed > 19){ 
    icurcase = 6;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-fixed_width,horizontal_center);
      tft.print(intstring[0]);
      tft.setCursor(vertical_center,horizontal_center);
      tft.print(intstring[1]);
    }
    else{
      if(prevstring[0] != intstring[0]){
        tft.fillRect(vertical_center-fixed_width, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center-fixed_width,horizontal_center);
        tft.print(intstring[0]);
        tft.fillRect(vertical_center, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center,horizontal_center);
        tft.print(intstring[1]);
      }
      else if(prevstring[1] != intstring[1]){
        tft.fillRect(vertical_center, horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
        tft.setCursor(vertical_center,horizontal_center);
        tft.print(intstring[1]);
      }   
    }
  }
  else{ 
    icurcase = 7;
    if(ipreviouscase != icurcase){
      tft.fillRect(vertical_center-(fixed_width*2)-num_1_offset, horizontal_center-fixed_height, (fixed_width*4)+10, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2),horizontal_center);
      tft.print(intstring[0]);
    }
    else if(prevstring[0] != intstring[0]){
      tft.fillRect(vertical_center-(fixed_width/2), horizontal_center-fixed_height, fixed_width, fixed_height, inner_circles);
      tft.setCursor(vertical_center-(fixed_width/2),horizontal_center);
      tft.print(intstring[0]);
    }
  }
  ipreviouscase = icurcase;
  iprevious_mph = ispeed;
}


//reads the OBD-II on board MEMS sensor and converts it to degrees F
//the degrees symbol is not supported in the graphics library so 
//I used a small "o" character instead
void print_int_cabin_temperature(int itemperature, int x, int y){
  char intstring[4] = " ";
  String tempstr = String(itemperature);
  tempstr.toCharArray(intstring,4);
  tft.graphicsMode();
  tft.setTextSize(1);
  tft.setFont(&FreeSansBold18pt7b);
  tft.fillRect(x-15, y-30, 125+15, 40, bottom_middle);
  tft.setCursor(x,y);
  
  if(itemperature >99){
    tft.setCursor(x-15,y);
    tft.print(intstring);
    tft.setFont(&FreeSans12pt7b);
    tft.setCursor(x+23+25,y-15);
    tft.print("o");
    tft.setFont(&FreeSansBold18pt7b);
    tft.setCursor(x+38+25,y);
    tft.print("F");
  }
  else if(itemperature > 9){
    tft.setCursor(x-5,y);
    tft.print(intstring);
    tft.setFont(&FreeSans12pt7b);
    tft.setCursor(x+23+20-5,y-15);
    tft.print("o");
    tft.setFont(&FreeSansBold18pt7b);
    tft.setCursor(x+38+20-5,y);
    tft.print("F");
  }
  else{
    tft.setCursor(x+5,y);
    tft.print(intstring);
    tft.setFont(&FreeSans12pt7b);
    tft.setCursor(x+23+5,y-15);
    tft.print("o");
    tft.setFont(&FreeSansBold18pt7b);
    tft.setCursor(x+38+5,y);
    tft.print("F");
  }
}

//This prints out the engine coolant temperature
//the degrees symbol is not supported in the graphics library so 
//I used a small "o" character instead
void print_int_coolant_temp(int itemperature, int x, int y){
  char intstring[4] = " ";
  String tempstr = String(itemperature);
  tempstr.toCharArray(intstring,4);
  tft.graphicsMode();
  tft.setTextSize(1);
  tft.setFont(&FreeSansBold12pt7b);
  tft.fillRect(x, y-20, 100, 25, bottom_background);
  tft.setCursor(x,y);
  
  if(itemperature >99){
    tft.setCursor(x,y);
    tft.print(intstring);
    tft.setFont(&FreeSans9pt7b);
    tft.setCursor(x+42,y-10);
    tft.print("o");
    tft.setFont(&FreeSansBold12pt7b);
    tft.setCursor(x+52,y);
    tft.print("F");
  }
  else if(itemperature > 9){
    tft.setCursor(x,y);
    tft.print(intstring);
    tft.setFont(&FreeSans9pt7b);
    tft.setCursor(x+30,y-10);
    tft.print("o");
    tft.setFont(&FreeSansBold12pt7b);
    tft.setCursor(x+40,y);
    tft.print("F");
  }
  else{
    tft.setCursor(x,y);
    tft.print(intstring);
    tft.setFont(&FreeSans9pt7b);
    tft.setCursor(x+15,y-10);
    tft.print("o");
    tft.setFont(&FreeSansBold12pt7b);
    tft.setCursor(x+25,y);
    tft.print("F");
  }
}

//This function reads the temperature from a DS18S20 temp sensor
void read_temperature(){
  int HighByte, LowByte, TReading, SignBit, Tc_100, Whole, Fract;
  byte i;
  byte present = 0;
  byte data[12];
  byte addr[8];

  if ( !ds.search(addr)) {
      ds.reset_search();
      return;
  }
  ds.reset();
  ds.select(addr);
  ds.write(0x44,1);

  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE); 
 
 
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);

  for ( i = 0; i < 9; i++) { 
    data[i] = ds.read();
  }
  LowByte = data[0];
  HighByte = data[1];
  TReading = (HighByte << 8) + LowByte;
  SignBit = TReading & 0x8000;  
  if (SignBit)
  {
    TReading = (TReading ^ 0xffff) + 1;
  }
  Tc_100 = (6 * TReading) + TReading / 4; 
  Whole = Tc_100 / 100; 
  Fract = Tc_100 % 100;
  if (SignBit)
  {
     Whole = (-1)*Whole;
  }
  float ftemp = (((float)Whole) *1.8) + 32.0; //convert to F
  icabin_temperature = (int) ftemp;
  //Serial.println(icabin_temperature);
}

//This function is the included function in the OBD library example
//with a modified pidlist to only get the PIDs that I wanted to display
//there are more available to read
void readPIDs()
{   //                               104=27            105=77         10C=802    10D=0      
    static const byte pidlist[] = {PID_ENGINE_LOAD, PID_COOLANT_TEMP, PID_RPM, PID_SPEED};
    for (byte i = 0; i < sizeof(pidlist) / sizeof(pidlist[0]); i++) {
        byte pid = pidlist[i];
        bool valid = obd.isValidPID(pid);
        //Serial.print((int)pid | 0x100, HEX);
        //Serial.print('=');
        if (valid) {
            int value;
            if (obd.readPID(pid, value)) {
              //Serial.print(value);
              switch (i){
                case 0:
                  iengine_load = value;
                  break;
                case 1:
                  icoolant_temp = (int)(((float)value)*1.8 + 32.0); //convert to F
                  break;
                case 2:
                  irpm = value;
                  break;
                case 3:
                  ispeed_mph = (int)(((float)value)*0.62);  //convert to mph
                  break;    
              }
            }
        }
     }
}

